@@ -330,16 +330,19 @@ impl Drop for DbBackend {
 pub struct Db<K: ?Sized, V: ?Sized, C> {
     key: PhantomData<fn(&K) -> &K>,
     value: PhantomData<fn(&V) -> &V>,
-    constraint: PhantomData<fn(C) -> C>,
+    constraint: C,
     backend: ArcByAddr<DbBackend>,
 }
 
-impl<K: ?Sized, V: ?Sized, C> Clone for Db<K, V, C> {
+impl<K: ?Sized, V: ?Sized, C> Clone for Db<K, V, C>
+where
+    C: Constraint,
+{
     fn clone(&self) -> Self {
         Db {
             key: PhantomData,
             value: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             backend: self.backend.clone(),
         }
     }
@@ -417,7 +420,10 @@ pub struct Cursor<K: ?Sized, V: ?Sized, C> {
     backend: ArcByAddr<CursorBackend>,
 }
 
-impl<K: ?Sized, V: ?Sized, C> Clone for Cursor<K, V, C> {
+impl<K: ?Sized, V: ?Sized, C> Clone for Cursor<K, V, C>
+where
+    C: Constraint,
+{
     fn clone(&self) -> Self {
         Cursor {
             db: self.db.clone(),
@@ -474,12 +480,13 @@ pub struct EnvBuilder<P> {
 }
 
 /// Constraints on database (type argument `C` to [`DbOptions`] and [`Db`])
-pub trait Constraint: 'static {
+pub trait Constraint: Copy + 'static {
     /// Duplicate keys allowed?
     const DUPLICATE_KEYS: bool;
 }
 
 /// Type argument to [`DbOptions`] and [`Db`] indicating unique keys
+#[derive(Clone, Copy)]
 pub struct KeysUnique;
 
 impl Constraint for KeysUnique {
@@ -487,6 +494,7 @@ impl Constraint for KeysUnique {
 }
 
 /// Type argument to [`DbOptions`] and [`Db`] indicating non-unique keys
+#[derive(Clone, Copy)]
 pub struct KeysDuplicate;
 
 impl Constraint for KeysDuplicate {
@@ -539,7 +547,7 @@ impl Constraint for KeysDuplicate {
 pub struct DbOptions<K: ?Sized, V: ?Sized, C, N> {
     key: PhantomData<fn(&K) -> &K>,
     value: PhantomData<fn(&V) -> &V>,
-    constraint: PhantomData<fn(C) -> C>,
+    constraint: C,
     lmdb_flags: LmdbFlags,
     name: N,
 }
@@ -737,20 +745,23 @@ impl DbOptions<[u8], [u8], KeysUnique, ()> {
         Self {
             key: PhantomData,
             value: PhantomData,
-            constraint: PhantomData,
+            constraint: KeysUnique,
             lmdb_flags: 0,
             name: (),
         }
     }
 }
 
-impl<K: ?Sized, V: ?Sized, C> DbOptions<K, V, C, ()> {
+impl<K: ?Sized, V: ?Sized, C> DbOptions<K, V, C, ()>
+where
+    C: Constraint,
+{
     /// Clear `dupsort` option (also clears `reversedup` option)
     pub const fn keys_unique(mut self) -> DbOptions<K, V, KeysUnique, ()> {
         flag_set!(self, MDB_DUPSORT, false);
         flag_set!(self, MDB_REVERSEDUP, false);
         DbOptions {
-            constraint: PhantomData,
+            constraint: KeysUnique,
             key: PhantomData,
             value: PhantomData,
             lmdb_flags: self.lmdb_flags,
@@ -761,7 +772,7 @@ impl<K: ?Sized, V: ?Sized, C> DbOptions<K, V, C, ()> {
     pub const fn keys_duplicate(mut self) -> DbOptions<K, V, KeysDuplicate, ()> {
         flag_set!(self, MDB_DUPSORT, true);
         DbOptions {
-            constraint: PhantomData,
+            constraint: KeysDuplicate,
             key: PhantomData,
             value: PhantomData,
             lmdb_flags: self.lmdb_flags,
@@ -777,7 +788,7 @@ impl<K: ?Sized, V: ?Sized, C> DbOptions<K, V, C, ()> {
         DbOptions {
             key: PhantomData,
             value: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             lmdb_flags: self.lmdb_flags,
             name: self.name,
         }
@@ -792,14 +803,17 @@ impl<K: ?Sized, V: ?Sized, C> DbOptions<K, V, C, ()> {
         DbOptions {
             value: PhantomData,
             key: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             lmdb_flags: self.lmdb_flags,
             name: self.name,
         }
     }
 }
 
-impl<K: ?Sized, V: ?Sized, C, N> DbOptions<K, V, C, N> {
+impl<K: ?Sized, V: ?Sized, C, N> DbOptions<K, V, C, N>
+where
+    C: Constraint,
+{
     /// Get `reversekey` option
     pub const fn get_reversekey(&self) -> bool {
         flag_get!(self, MDB_REVERSEKEY)
@@ -828,7 +842,7 @@ impl<K: ?Sized, V: ?Sized, C, N> DbOptions<K, V, C, N> {
         DbOptions {
             value: PhantomData,
             key: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             lmdb_flags: self.lmdb_flags,
             name: None,
         }
@@ -842,7 +856,7 @@ impl<K: ?Sized, V: ?Sized, C, N> DbOptions<K, V, C, N> {
         DbOptions {
             value: PhantomData,
             key: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             lmdb_flags: self.lmdb_flags,
             name: Some(name),
         }
@@ -855,7 +869,7 @@ impl<K: ?Sized, V: ?Sized, C, N> DbOptions<K, V, C, N> {
         DbOptions {
             value: PhantomData,
             key: PhantomData,
-            constraint: PhantomData,
+            constraint: self.constraint,
             lmdb_flags: self.lmdb_flags,
             name: (),
         }
@@ -907,7 +921,7 @@ pub trait Env {
     /// compatible options.
     unsafe fn open_dbs<'a, K, V, C>(
         &self,
-        metas: impl IntoIterator<
+        options: impl IntoIterator<
             IntoIter = impl Iterator<Item = &'a DbOptions<K, V, C, Option<CString>>> + ExactSizeIterator,
         >,
     ) -> Result<Vec<Db<K, V, C>>, io::Error>
@@ -1052,7 +1066,7 @@ impl EnvBackend {
             dbs.push(Db {
                 key: PhantomData,
                 value: PhantomData,
-                constraint: PhantomData,
+                constraint: option.constraint,
                 backend: ArcByAddr::new(db_backend),
             });
         }
@@ -1309,6 +1323,7 @@ impl<'a> TxnBackend<'a> {
     where
         K: ?Sized,
         V: ?Sized,
+        C: Constraint,
     {
         db.backend.assert_env_backend(&self.env_backend);
         unsafe {
